;;; package -- Summary
;;; Init.el

;;; Commentary:
;;; My Init.el

;;; Code:
(setq inhibit-startup-screen t )	; inhibit useless and old-school startup screen
(setq tab-width 4)
(setq initial-scratch-message "Welcome in Emacs")
(setq vc-follow-symlinks t)
(setq backup-directory-alist '((".*" . "/tmp")))
(menu-bar-mode -1)
(toggle-scroll-bar -1)
(tool-bar-mode -1)

(defun edit-init()
  "Edit 'init.el' quickly"
  (interactive)
  (find-file user-init-file))

;; Package Management
(require 'package)
(setq package-archives
   '(("org"       . "http://orgmode.org/elpa/")
    ("gnu"       . "http://elpa.gnu.org/packages/")
    ("melpa"     . "https://melpa.org/packages/")
    ("marmalade" . "http://marmalade-repo.org/packages/")))

(package-initialize)

;; Why we use this line
;; https://www.reddit.com/r/emacs/comments/1rdstn/set_packageenableatstartup_to_nil_for_slightly/
(setq package-enable-at-startup nil)

(unless (package-installed-p 'use-package)
  (package-refresh-contents)
  (package-install 'use-package))

;; This is so our PATH gets read in correctly
(when (memq window-system '(mac ns x))
  (exec-path-from-shell-initialize))

(eval-when-compile (require 'use-package))
(custom-set-variables
 ;; custom-set-variables was added by Custom.
 ;; If you edit it by hand, you could mess it up, so be careful.
 ;; Your init file should contain only one such instance.
 ;; If there is more than one, they won't work right.
 '(evil-want-Y-yank-to-eol t)
 '(package-selected-packages
   (quote
    (sly intero haskell-mode company-quickhelp ivy color-theme-sanityinc-tomorrow zenburn-theme sublime-themes flycheck exec-path-from-shell flycheck-rust evil-mc evil-snipe evil-surround general evil-easymotion vimrc-mode evil-commentary markdown-preview-mode markdown-mode linum-relative company-mode company which-key omnisharp use-package evil)))
 '(send-mail-function (quote smtpmail-send-it)))
(custom-set-faces
 ;; custom-set-faces was added by Custom.
 ;; If you edit it by hand, you could mess it up, so be careful.
 ;; Your init file should contain only one such instance.
 ;; If there is more than one, they won't work right.
 )

(setq use-package-always-insure t)

(load-theme 'sanityinc-tomorrow-eighties t)

(use-package evil
  :init
  (setq evil-want-C-u-scroll t)
  :config
  (evil-mode t))

(use-package general
  :init
  (general-evil-setup)
  (setq leader "SPC")
  (general-nmap :prefix leader "fi" 'edit-init)
  (general-nmap :prefix leader "h" 'help-command)
  (general-nmap :prefix leader "s" 'isearch-forward)
  (general-nmap "Y" 'evil-yank)
  (general-nmap "C-h" 'evil-window-left)
  (general-nmap "C-j" 'evil-window-down)
  (general-nmap "C-k" 'evil-window-up)
  (general-nmap "C-l" 'evil-window-right)
  (general-nmap "C-s" 'save-buffer))

(electric-pair-mode 1)

(use-package linum-relative
  :init
  (setq linum-relative-current-symbol "")
  (linum-relative-global-mode))

(setq inferior-lisp-program "/usr/bin/sbcl")
(setq slime-contribs '(slime-fancy slime-company))

;; (use-package ivy
;;   :config
;;   (setq ivy-re-builders-alist
;; 	'((t . ivy--regex-plus)))
;;   :init (ivy-mode))

(use-package recentf-mode
  :init
  (setq recentf-max-menu-items 50)
  (recentf-mode 1)
  :config
  (add-hook 'find-file-hook 'recentf-save-list)
  :general
  (general-nmap :prefix leader "fr" 'recentf-open-files))

(use-package avy
  :config
    (setq avy-timeout-seconds 0.23)
    (setq avy-keys '(?a ?s ?d ?f ?w ?e ?r ?u ?i ?o ?h ?j ?k ?l))
    (setq avy-background t)
    (setq avy-style 'at-full)
  :general
  (general-nmap ";" 'avy-goto-char-timer)
  (general-nmap ",w" 'avy-goto-word-0-below)
  (general-nmap ",b" 'avy-goto-word-0-above))

(use-package evil-snipe
  :init
  (setq evil-snipe-override-evil-repeat-keys nil)
  (evil-snipe-override-mode))

(use-package evil-surround
  :config
  (global-evil-surround-mode 1))

(use-package evil-commentary :init (evil-commentary-mode))

(use-package which-key
  :init
  (setq which-key-idle-delay 0.4)
  (setq which-key-add-column-padding 3)
  :config
  (which-key-mode)
  (which-key-setup-side-window-bottom))

(use-package company
  :init
  (setq company-idle-delay 0.05)
  (setq company-minimum-prefix-length 2)
  (add-hook 'after-init-hook 'global-company-mode))

(use-package flycheck
  :init
  (global-flycheck-mode))

(use-package racer
  :after rust-mode
  :config
  (setq racer-rust-src-path "~/.rustup/toolchains/stable-x86_64-unknown-linux-gnu/lib/rustlib/src/rust/src")
  :init
  (add-hook 'rust-mode-hook 'racer-mode)
  (add-hook 'rust-mode-hook 'cargo-minor-mode)
  (add-hook 'flycheck-mode-hook #'flycheck-rust-setup)
  (setq flycheck-checker 'rust-cargo))

(use-package intero
  :after haskell-mode
  :init
  (add-hook 'haskell-mode-hook 'intero-mode))

(use-package omnisharp
  :after csharp-mode
  :preface
  (progn
    (defun my/configure-omnisharp ()
      (omnisharp-mode)
      (add-to-list 'company-backends #'company-omnisharp)
      (company-mode)
      (local-set-key (kbd "C-c C-c") #'recompile)
      (setq omnisharp-company-match-type 'company-match-server)))
  :init
  (progn
    (add-hook 'csharp-mode-hook #'my/configure-omnisharp))
  :config
  (progn
    (bind-key "C-c r r" #'omnisharp-run-code-action-refactoring omnisharp-mode-map)))

;;; init.el ends here
